@@ -13,3 +13,14 @@ vec vec_init(const vec& orig)
 {
 	return vec(orig.n_elem, fill::zeros);
 }
+
+
+
+ivec ivec2_iter(const ivec& v)
+{
+	const int n = v.n_elem;
+	ivec out(n+1);
+	out[0] = 0;
+	out.tail(n) = cumsum(v);
+	return out;
+}
